@@ -21,12 +21,31 @@ pub const MIN_CLIENT_VERSION: u32 = 1;
 pub const HEADER_SESSION: &str = "X-GR-Session";
 pub const CONTENT_TYPE: &str = "x-content-type";
 
+//TODO: use everywhere
+pub type UserId = String;
+
+/// session validation responses
+pub enum SessionValidationResult {
+	/// returns `user_id` belonging to the session
+	Ok { user_id: UserId },
+	/// session is known but was supercedet
+	Invalid,
+	/// unknown or timedout session
+	Unknown,
+}
+
 pub struct UserLogin {}
 
 #[async_trait]
 pub trait UserLoginEvents: Send + Sync {
-	async fn on_login(&self, _user: &User);
-	async fn on_register(&self, _user: &User);
+	async fn on_login(
+		&self,
+		_user: &User,
+	) -> Result<(), error::Error>;
+	async fn on_register(
+		&self,
+		_user: &User,
+	) -> Result<(), error::Error>;
 }
 
 pub struct UserLoginResource {
@@ -60,6 +79,21 @@ impl UserLoginResource {
 		self.ipdb = Some(ipdb);
 	}
 
+	pub async fn validate_session(
+		&self,
+		session: &str,
+	) -> SessionValidationResult {
+		match self.sessions.get(session).await {
+			Some(session) if session.valid => {
+				SessionValidationResult::Ok {
+					user_id: session.user_id,
+				}
+			}
+			Some(_) => SessionValidationResult::Invalid,
+			None => SessionValidationResult::Unknown,
+		}
+	}
+
 	async fn country_from_ip(
 		&self,
 		ip: Option<String>,
@@ -106,15 +140,16 @@ impl UserLoginResource {
 				//TODO: add last_login to User, update last login here
 
 				user.country = self.country_from_ip(ip).await;
-				user.language = login_request.clientLanguage;
+				user.language =
+					string_to_option(login_request.clientLanguage);
 
 				user.version += 1;
 				if self.users.save_user(&user).await.is_err() {
-					tracing::error!("user save error")
+					tracing::error!("user save error");
 				}
 
 				if let Some(events) = self.events.as_ref() {
-					events.on_login(&user).await;
+					events.on_login(&user).await?;
 				}
 
 				tracing::info!("user succesfully logged in");
@@ -129,7 +164,7 @@ impl UserLoginResource {
 			}
 		}
 
-		Err(error::Error::IoError(std::io::Error::new(
+		Err(error::Error::Io(std::io::Error::new(
 			std::io::ErrorKind::NotFound,
 			"failed to retrieve user",
 		)))
@@ -153,6 +188,7 @@ impl UserLoginResource {
 		}
 
 		let country = self.country_from_ip(ip).await;
+		let client_language = string_to_option(client_language);
 
 		let mut new_user = User::new(country, client_language);
 		let session = self
@@ -165,7 +201,7 @@ impl UserLoginResource {
 		self.users.save_user(&new_user).await?;
 
 		if let Some(events) = self.events.as_ref() {
-			events.on_register(&new_user).await;
+			events.on_register(&new_user).await?;
 		}
 
 		tracing::info!("registered user: {}", &new_user.id);
@@ -192,33 +228,27 @@ impl CustomModule for UserLogin {
 	fn create_filter<S: ModuleResources<Self>>(
 		server: std::sync::Arc<S>,
 	) -> warp::filters::BoxedFilter<(Box<dyn warp::Reply>,)> {
-		let server2 = server.clone();
+		let userlogin = warp::any().map(move || {
+			let (resource, _) =
+				server.clone().get_server_resources().pluck();
+			resource
+		});
 
 		let register_filter = warp::path!("user" / "register")
 			.and(warp::post())
 			.and(warp::header::optional::<String>("X-Forwarded-For"))
 			.and(warp::addr::remote())
 			.and(pbwarp::protobuf_body::<schema::RegisterRequest>())
-			.and(warp::any().map(move || {
-				let (resource, _) =
-					server2.get_server_resources().pluck();
-				resource
-			}))
+			.and(userlogin.clone())
 			.and(warp::header::optional::<String>(CONTENT_TYPE))
 			.and_then(register_filter_fn);
-		// .with(warp::wrap_fn(|a| a));
-		// .with(warp::wrap_fn(pbwarp::wrapper)); //wrap_fn requires the error to be infallible
 
 		let login_filter = warp::path!("user" / "login")
 			.and(warp::post())
 			.and(warp::header::optional::<String>("X-Forwarded-For"))
 			.and(warp::addr::remote())
 			.and(pbwarp::protobuf_body::<schema::LoginRequest>())
-			.and(warp::any().map(move || {
-				let (resource, _) =
-					server.clone().get_server_resources().pluck();
-				resource
-			}))
+			.and(userlogin)
 			.and(warp::header::optional::<String>(CONTENT_TYPE))
 			.and_then(login_filter_fn);
 
@@ -318,14 +348,14 @@ pub fn session_filter(
 		resource: Arc<UserLoginResource>,
 		session: String,
 	) -> Result<String, Rejection> {
-		match resource.sessions.get(&session).await {
-			Some(session) if session.valid => Ok(session.user_id),
-			Some(_) => {
+		match resource.validate_session(&session).await {
+			SessionValidationResult::Ok { user_id } => Ok(user_id),
+			SessionValidationResult::Invalid => {
 				Err(warp::reject::custom(SessionFailure::Invalid))
 			}
-			None => Err(warp::reject::custom(
-				SessionFailure::SessionNotFound,
-			)),
+			SessionValidationResult::Unknown => Err(
+				warp::reject::custom(SessionFailure::SessionNotFound),
+			),
 		}
 	}
 
@@ -340,6 +370,14 @@ pub const fn is_valid_version(client_version: u32) -> bool {
 	client_version >= MIN_CLIENT_VERSION
 }
 
+fn string_to_option(string: String) -> Option<String> {
+	if string.is_empty() {
+		None
+	} else {
+		Some(string)
+	}
+}
+
 #[cfg(test)]
 mod tests {
 	use crate::{
@@ -675,7 +713,7 @@ mod tests {
 
 		let db_user = users.clone().get_user(&user.id).await.unwrap();
 		assert_eq!(db_user.id, user.id);
-		assert_eq!(db_user.language, "en-CA");
+		assert_eq!(db_user.language, Some("en-CA".to_string()));
 		assert_eq!(db_user.session, Some(session.clone()));
 		assert_eq!(db_user.secret, user.secret);
 		assert_eq!(
@@ -735,7 +773,7 @@ mod tests {
 		let db_session = sessions.get(&session).await.unwrap();
 		let db_user =
 			users.get_user(&db_session.user_id).await.unwrap();
-		assert_eq!(db_user.language, "en-CA");
+		assert_eq!(db_user.language, Some("en-CA".to_string()));
 		assert_eq!(db_user.session, Some(session.clone()));
 	}
 
